<?php

namespace WPSpeedo_Team;

use  WP_Query, WP_Error ;
if ( !defined( 'ABSPATH' ) ) {
    exit;
}
class Helpers
{
    static function get_posts_meta_cache_key( $meta_key, $post_type = null )
    {
        if ( empty($post_type) ) {
            $post_type = self::post_type_name();
        }
        return sprintf( 'wps--meta-vals--%s_%s', $post_type, $meta_key );
    }
    
    static function update_posts_meta_vals( $meta_key, $post_type = null )
    {
        if ( empty($post_type) ) {
            $post_type = self::post_type_name();
        }
        $cache_key = self::get_posts_meta_cache_key( $meta_key, $post_type );
        delete_transient( $cache_key );
        return self::get_posts_meta_vals( $meta_key, $post_type );
    }
    
    static function update_all_posts_meta_vals( $meta_fields = array(), $post_type = null )
    {
        $meta_fields = ( !empty($meta_fields) ? $meta_fields : [ '_ribbon' ] );
        if ( empty($post_type) ) {
            $post_type = self::post_type_name();
        }
        foreach ( $meta_fields as $meta_key ) {
            self::update_posts_meta_vals( $meta_key, $post_type );
        }
    }
    
    static function get_posts_meta_vals( $meta_key, $post_type = null )
    {
        global  $wpdb ;
        if ( empty($post_type) ) {
            $post_type = self::post_type_name();
        }
        $cache_key = self::get_posts_meta_cache_key( $meta_key, $post_type );
        $cache_data = get_transient( $cache_key );
        if ( $cache_data !== false ) {
            return $cache_data;
        }
        $results = $wpdb->get_results( $wpdb->prepare( "\n\t\t\tselect META.meta_value\n\t\t\tfrom {$wpdb->postmeta} AS META\n\t\t\tINNER JOIN {$wpdb->posts} AS POST\n\t\t\tON META.post_id = POST.ID\n\t\t\twhere POST.post_type = %s AND\n\t\t\tPOST.post_status = 'publish' AND\n\t\t\tMETA.meta_key = %s;\n\t\t", $post_type, $meta_key ) );
        
        if ( !empty($results) ) {
            $results = wp_list_pluck( $results, 'meta_value' );
            $results = array_values( array_unique( $results ) );
            set_transient( $cache_key, $results, MINUTE_IN_SECONDS * 10 );
            return $results;
        }
        
        return [];
    }
    
    static function get_posts( $query_args = array() )
    {
        $args = [
            'posts_per_page' => -1,
        ];
        $args = array_merge( $args, $query_args );
        $args = (array) apply_filters( 'wpspeedo_team/query_params', $args );
        $args['post_type'] = Helpers::post_type_name();
        return new WP_Query( $args );
    }
    
    static function get_item_data( $data_key, $post_id = null )
    {
        if ( empty($post_id) ) {
            $post_id = get_the_ID();
        }
        $meta_fields = [
            '_experience',
            '_company',
            '_skills',
            '_designation',
            '_telephone',
            '_email',
            '_website',
            '_social_links',
            '_ribbon',
            '_mobile',
            '_color'
        ];
        $taxonomies = self::get_taxonomies();
        
        if ( in_array( $data_key, $meta_fields ) ) {
            return get_post_meta( $post_id, $data_key, true );
        } else {
            if ( in_array( $data_key, $taxonomies ) ) {
                return get_the_terms( $post_id, $data_key );
            }
        }
        
        return false;
    }
    
    static function load_template( $template_name )
    {
        $template_folder = (string) apply_filters( 'wpspeedo_team/template/folder', 'wpspeedo-team' );
        $template_folder = '/' . trailingslashit( ltrim( $template_folder, '/\\' ) );
        // Load from current theme
        $template_path = get_stylesheet_directory() . $template_folder . $template_name;
        if ( file_exists( $template_path ) ) {
            return $template_path;
        }
        // Load from parent theme if not found in child theme
        
        if ( is_child_theme() ) {
            $template_path = get_template_directory() . $template_folder . $template_name;
            if ( file_exists( $template_path ) ) {
                return $template_path;
            }
        }
        
        // Load from plugin site
        $template_path = WPS_TEAM_PATH . 'templates/' . $template_name;
        if ( file_exists( $template_path ) ) {
            return $template_path;
        }
        return new WP_Error( 'wpspeedo_team/template/not_found', __( 'Template file is not found', 'wpspeedo-team' ) );
    }
    
    static function get_temp_settings()
    {
        $temp_key = self::get_shortcode_preview_key();
        if ( $temp_key ) {
            return get_transient( $temp_key );
        }
    }
    
    static function is_shortcode_preview()
    {
        return (bool) (!empty($_REQUEST['wps_shortcode_preview']));
    }
    
    static function get_shortcode_preview_key()
    {
        return ( self::is_shortcode_preview() ? sanitize_text_field( $_REQUEST['wps_shortcode_preview'] ) : null );
    }
    
    public static function render_html_attributes( array $attributes )
    {
        $rendered_attributes = [];
        foreach ( $attributes as $attribute_key => $attribute_values ) {
            if ( is_array( $attribute_values ) ) {
                $attribute_values = implode( ' ', $attribute_values );
            }
            $rendered_attributes[] = sprintf( '%1$s="%2$s"', $attribute_key, esc_attr( $attribute_values ) );
        }
        return implode( ' ', $rendered_attributes );
    }
    
    public static function max_letters( $text, $limit )
    {
        if ( strlen( $text ) > $limit ) {
            $text = substr( $text, 0, $limit ) . '...';
        }
        return $text;
    }
    
    public static function get_brnad_name( $icon )
    {
        return str_replace( [ 'fab fa-', 'far fa-', 'fas fa-' ], '', $icon );
    }
    
    public static function sanitize_phone_number( $phone )
    {
        return preg_replace( '/[^0-9\\-\\_\\+]*/', '', $phone );
    }
    
    public static function default_settings()
    {
        return [
            'enable_archive'       => true,
            'post_type_slug'       => 'wps-members',
            'enable_group_archive' => false,
            'group_slug'           => 'wps-members-group',
            'member_plural_name'   => __( 'Team Members', 'wpspeedo-team' ),
            'member_single_name'   => __( 'Member', 'wpspeedo-team' ),
            'group_plural_name'    => __( 'Team Groups', 'wpspeedo-team' ),
            'group_single_name'    => __( 'Group', 'wpspeedo-team' ),
        ];
    }
    
    public static function get_settings()
    {
        $defaults = self::default_settings();
        $settings = (array) get_option( self::get_option_name(), $defaults );
        $settings = array_merge( $defaults, $settings );
        $fields = [
            'post_type_slug',
            'group_slug',
            'member_plural_name',
            'member_single_name',
            'group_plural_name',
            'group_single_name'
        ];
        foreach ( $fields as $field ) {
            if ( empty($settings[$field]) ) {
                $settings[$field] = $defaults[$field];
            }
        }
        return $settings;
    }
    
    public static function get_setting( $key )
    {
        $settings = self::get_settings();
        if ( array_key_exists( $key, $settings ) ) {
            return $settings[$key];
        }
        return null;
    }
    
    public static function has_archive()
    {
        return wp_validate_boolean( self::get_setting( 'enable_archive' ) );
    }
    
    public static function has_group_archive()
    {
        return wp_validate_boolean( self::get_setting( 'enable_group_archive' ) );
    }
    
    public static function get_taxonomies()
    {
        return [ 'wps-team-group' ];
    }
    
    public static function archive_enabled_taxonomies()
    {
        return [ 'wps-team-group' ];
    }
    
    public static function post_type_name()
    {
        return 'wps-team-members';
    }
    
    public static function group_taxonomy_name()
    {
        return 'wps-team-group';
    }
    
    public static function get_option_name()
    {
        return 'wps_team_members';
    }
    
    public static function get_archive_slug()
    {
        return self::get_setting( 'post_type_slug' );
    }
    
    public static function get_group_archive_slug()
    {
        return self::get_setting( 'group_slug' );
    }
    
    public static function flush_rewrite_rules()
    {
        delete_option( self::rewrite_flush_key() );
    }
    
    public static function rewrite_flush_key()
    {
        return 'wps-rewrite--flushed';
    }
    
    public static function get_plugin_icon()
    {
        return WPS_TEAM_URL . '/images/icon.svg';
    }
    
    public static function get_pro_label()
    {
        return __( '(Pro) - ', 'wpspeedo-team' );
    }
    
    public static function get_options_display_type()
    {
        $options = [ [
            'label' => 'Grid',
            'value' => 'grid',
        ], [
            'label' => 'Carousel',
            'value' => 'carousel',
        ], [
            'label' => __( 'Filter' ),
            'value' => 'filter',
        ] ];
        return $options;
    }
    
    public static function get_options_theme()
    {
        $options = [
            [
            'label' => __( 'Theme One' ),
            'value' => 'theme-one',
        ],
            [
            'label' => __( 'Theme Two' ),
            'value' => 'theme-two',
        ],
            [
            'label' => __( 'Theme Three' ),
            'value' => 'theme-three',
        ],
            [
            'label' => __( 'Theme Four' ),
            'value' => 'theme-four',
        ],
            [
            'label' => __( 'Theme Five' ),
            'value' => 'theme-five',
        ],
            [
            'label' => __( 'Theme Six' ),
            'value' => 'theme-six',
        ]
        ];
        return $options;
    }
    
    public static function get_options_card_action()
    {
        $options = [
            [
            'label' => __( 'None' ),
            'value' => 'none',
        ],
            [
            'label' => __( 'Single Page' ),
            'value' => 'single-page',
        ],
            [
            'disabled' => true,
            'label'    => __( 'Modal' ),
            'value'    => 'modal',
        ],
            [
            'disabled' => true,
            'label'    => __( 'Side Panel' ),
            'value'    => 'side-panel',
        ],
            [
            'disabled' => true,
            'label'    => __( 'Expand' ),
            'value'    => 'expand',
        ]
        ];
        return $options;
    }
    
    public static function get_options_orderby()
    {
        $options = [
            [
            'label' => __( 'ID' ),
            'value' => 'ID',
        ],
            [
            'label' => __( 'Title' ),
            'value' => 'title',
        ],
            [
            'label' => __( 'Date' ),
            'value' => 'date',
        ],
            [
            'label' => __( 'Random' ),
            'value' => 'rand',
        ],
            [
            'label' => __( 'Modified' ),
            'value' => 'modified',
        ],
            [
            'disabled' => true,
            'label'    => __( 'Custom Order' ),
            'value'    => 'menu_order',
        ]
        ];
        return $options;
    }
    
    public static function get_post_term_slugs( $term_name, $separator = ' ' )
    {
        global  $post ;
        $terms = get_the_terms( $post->ID, $term_name );
        
        if ( !empty($terms) && !is_wp_error( $terms ) ) {
            $terms = implode( $separator, wp_list_pluck( $terms, 'slug' ) );
            return $terms;
        }
        
        return '';
    }
    
    public static function get_groups()
    {
        return get_terms( [
            'taxonomy'   => self::group_taxonomy_name(),
            'orderby'    => 'name',
            'order'      => 'ASC',
            'hide_empty' => false,
        ] );
    }
    
    public static function get_control_options( $control_id )
    {
        $method = "get_options_{$control_id}";
        $options = self::$method();
        foreach ( $options as &$option ) {
            if ( array_key_exists( 'disabled', $option ) ) {
                $option['label'] = self::get_pro_label() . $option['label'];
            }
        }
        return $options;
    }
    
    public static function get_free_themes()
    {
        return [
            'theme-one',
            'theme-two',
            'theme-three',
            'theme-four',
            'theme-five',
            'theme-six'
        ];
    }
    
    public static function get_active_themes()
    {
        $themes = self::get_free_themes();
        return $themes;
    }
    
    public static function get_wps_team( $shortcode_id )
    {
        return do_shortcode( sprintf( '[wpspeedo-team id=%d]', $shortcode_id ) );
    }
    
    public static function get_top_label_menu()
    {
        return 'edit.php?post_type=' . Helpers::post_type_name();
    }
    
    public static function get_pro_themes()
    {
        return [ 'theme-seven' ];
    }
    
    public static function get_all_active_themes( $options )
    {
        return array_merge( $options, self::get_pro_themes() );
    }
    
    public static function string_to_array( $terms = '' )
    {
        if ( empty($terms) ) {
            return [];
        }
        return (array) array_filter( explode( ',', $terms ) );
    }
    
    public static function get_demo_data_status( $demo_type = '' )
    {
        $status = [
            'post_data'      => wp_validate_boolean( get_option( 'wpspeedo_team_dummy_post_data_created' ) ),
            'shortcode_data' => wp_validate_boolean( get_option( 'wpspeedo_team_dummy_shortcode_data_created' ) ),
        ];
        if ( !empty($demo_type) && array_key_exists( $demo_type, $status ) ) {
            return $status[$demo_type];
        }
        return $status;
    }
    
    public static function get_installed_time()
    {
        $installed_time = get_option( '_wps_team_installed_time' );
        if ( !empty($installed_time) ) {
            return $installed_time;
        }
        $installed_time = time();
        update_option( '_wps_team_installed_time', $installed_time );
        return $installed_time;
    }
    
    public static function get_timestamp_diff( $old_time, $new_time = null )
    {
        if ( $new_time == null ) {
            $new_time = time();
        }
        return ceil( ($new_time - $old_time) / DAY_IN_SECONDS );
    }

}